<?php
/**
 * BSeller - B2W Companhia Digital
 *
 * DISCLAIMER
 *
 * Do not edit this file if you want to update this module for future new versions.
 *
 * @copyright     Copyright (c) 2017 B2W Companhia Digital. (http://www.bseller.com.br/)
 * @author        Luiz Tucillo <luiz.tucillo@e-smart.com.br>
 */

namespace B2W\SkyHub\Model\Repository\Product;

use B2W\SkyHub\Contract\Repository\Product\AttributeRepositoryInterface;
use B2W\SkyHub\Model\Entity\Product\Attribute\OptionEntity;
use B2W\SkyHub\Model\Entity\Product\AttributeEntity;
use B2W\SkyHub\Model\Resource\Collection;
use B2W\SkyHub\Model\Resource\Select;

/**
 * Class AttributeDbRepository
 * @package B2W\SkyHub\Model\Repository\Product
 */
class AttributeDbRepository implements AttributeRepositoryInterface
{
    /**
     * Prepare where to select
     * 
     * @return Select
     */
    protected function prepareWhere($filters, $select)
    {
        global $wpdb;

        if (!is_array($filters)) {
            return $select;
        }

        foreach ($filters as $where=>$val) {
            if (!empty($where) && !empty($val)) {
                $select->where($wpdb->prepare($where, $val));
            }
        }
        return $select;
    }

    protected function getSelectAll($filters = null)
    {
        global $wpdb;
        $select = $this->_getSelect();

        if (!is_array($filters)) {
            return $select;
        }

        $select = $this->prepareWhere($filters, $select);
        return $select->prepare();
    }
    /**
     * @param array $filters
     * @return Collection
     */
    public function all($filters = null)
    {

        $select = $this->getSelectAll($filters);
        
        $results    = $this->_prepareArrayAttributes($select);
        $collection = new Collection();

        foreach ($results as $result) {
            $attr = new AttributeEntity();
            $attr->setId($result['id'])
                ->setCode($result['code'])
                ->setLabel($result['label']);

            if (isset($result['options'])) {
                foreach ($result['options'] as $opt) {
                    $option = new OptionEntity();
                    $option->setId($opt['id'])
                        ->setCode($opt['code'])
                        ->setLabel($opt['label']);

                    $attr->addOption($option);
                }
            }

            $collection->addItem($attr);
        }

        return $collection;
    }

    /**
     * @param $id
     * @return AttributeEntity
     */
    public function one($id)
    {
        return $this->_getOne(['main_table.attribute_id = %s' => $id]);
    }

    public function getAttributeProduct($postId)
    {
        return $this->all(
            array(
                'term_relationships.object_id = %s' => $postId
            )
        );
    }

    /**
     * @param $code
     * @return AttributeEntity|mixed
     */
    public function code($code, $idProduct = null)
    {
        return $this->_getOne([
            'main_table.attribute_name = %s' => $code,
            'term_relationships.object_id = %s' => $idProduct
        ]);
    }

    /**
     * @param null $where
     * @param null $val
     * @return AttributeEntity
     */
    protected function _getOne($where = null)
    {
        global $wpdb;

        $select = $this->_getSelect();

        $select = $this->prepareWhere($where, $select);

        $results = current($this->_prepareArrayAttributes($select));

        $attr = new AttributeEntity();
        $attr->setId($results['id'])
            ->setCode($results['code'])
            ->setLabel($results['label']);

        if (isset($results['options'])) {
            foreach ($results['options'] as $opt) {
                $option = new OptionEntity();
                $option->setId($opt['id'])
                    ->setCode($opt['code'])
                    ->setLabel($opt['label']);

                $attr->addOption($option);
            }
        }

        return $attr;
    }

    /**
     * @param $query
     * @return array
     */
    protected function _prepareArrayAttributes($query)
    {
        global $wpdb;

        $query = $query instanceof Select ? $query->prepare() : $query;

        $results    = $wpdb->get_results($query);
        $attributes = array();

        foreach ($results as $result) {
            if (!isset($attributes[$result->attribute_id])) {
                $attributes[$result->attribute_id] = array(
                    'id'        => $result->attribute_id,
                    'code'      => $result->attribute_name,
                    'label'     => $result->attribute_label,
                    'options'   => array()
                );
            }

            if ($result->term_id) {
                $attributes[$result->attribute_id]['options'][$result->term_id] = array(
                    'id'    => $result->term_id,
                    'code'  => $result->slug,
                    'label' => $result->name
                );
            }
        }

        return $attributes;
    }

    /**
     * @return Select
     */
    protected function _getSelect()
    {
        $select = new Select();

        $select->column([
            'attribute_id',
            'attribute_name',
            'attribute_label',
            'terms.term_id',
            'terms.name',
            'terms.slug'        
        ]);

        $select->from('woocommerce_attribute_taxonomies', 'main_table');

        $select->join(
            'term_taxonomy',
            "term_taxonomy.taxonomy = CONCAT('pa_', main_table.attribute_name)",
            'term_taxonomy',
            'left'
        );

        $select->join(
            'terms',
            "terms.term_id = term_taxonomy.term_id",
            'terms',
            'left'
        );

        $select->join(
            'term_relationships',
            "terms.term_id = term_relationships.term_taxonomy_id",
            'term_relationships',
            'left'
        );

        return $select;
    }
}
