<?php
/**
 * BSeller - B2W Companhia Digital
 *
 * DISCLAIMER
 *
 * Do not edit this file if you want to update this module for future new versions.
 *
 * @copyright     Copyright (c) 2017 B2W Companhia Digital. (http://www.bseller.com.br/)
 * @author        Luiz Tucillo <luiz.tucillo@e-smart.com.br>
 */

namespace B2W\SkyHub\Model\Transformer;

use B2W\SkyHub\Model\Map\MapAttribute;

/**
 * Class DbToEntityAbstract
 * @package B2W\SkyHub\Model\Transformer
 */
abstract class DbToEntityAbstract
{
    /** @var \WP_Post */
    protected $_post            = null;
    /**
     * @var null
     */
    protected $_entity          = null;
    /**
     * @var null
     */
    protected $_parentEntity    = null;
    /**
     * @var null
     */
    protected $_meta            = null;
    /**
     * @var MapAttribute
     */
    protected $_attribute       = null;
    /**
     * @var DbToEntityAbstract
     */
    protected $_referenceClass  = null;

    /**
     * @return null
     */
    abstract protected function _getEntityInstance();

    /**
     * @return mixed
     */
    abstract protected function _getAttributeMap();

    /**
     * @return DbToEntityAbstract
     */
    public function getReferenceClass()
    {
        return $this->_referenceClass;
    }

    /**
     * @param DbToEntityAbstract $referenceClass
     */
    public function setReferenceClass($referenceClass)
    {
        $this->_referenceClass = $referenceClass;
    }

    /**
     * @param $post
     * @return $this
     */
    public function setPost($post)
    {
        $this->_post = $post;
        return $this;
    }

    /**
     * @param $entity
     * @return $this
     */
    public function setEntity($entity)
    {
        $this->_entity = $entity;
        return $this;
    }

    /**
     * @return $this
     */
    protected function _afterConvert()
    {
        return $this;
    }

    /**
     * @return null
     */
    protected function _getEntity()
    {
        if (is_null($this->_entity)) {
            $this->_entity = $this->_getEntityInstance();
        }

        return $this->_entity;
    }

    /**
     * @param $entity
     * @return $this
     */
    public function setParentEntity($entity)
    {
        $this->_parentEntity = $entity;
        return $this;
    }

    /**
     * @param $meta
     * @return $this
     */
    public function setMeta($meta)
    {
        $this->_meta = $meta;
        return $this;
    }

    /**
     * @param $attribute
     * @return $this
     */
    public function setAttribute(MapAttribute $attribute)
    {
        $this->_attribute = $attribute;
        return $this;
    }

    /**
     * @return null
     */
    public function getParentEntity()
    {
        return $this->_parentEntity;
    }


    /**
     * @return \B2W\SkyHub\Helper\App
     * @throws \B2W\SkyHub\Exception\Helper\HelperNotFound
     */
    protected function _helper()
    {
        return \App::helper('app');
    }

    /**
     * @return null
     * @throws \Exception
     */
    public function convert()
    {
        $this->_validate();

        /** @var MapAttribute $attribute */
        foreach ($this->_getAttributeMap() as $attribute) {

            $value = $attribute->getMapper('db_to_entity')
                ? $this->_fromMapper($attribute)
                : $this->_getPostValue($attribute);

            $this->_setValue($attribute, $value);
        }

        $entity = $this->_entity;

        $this->_reset();

        return $entity;
    }

    /**
     * @return $this
     */
    protected function _reset()
    {
        $this->_post            = null;
        $this->_entity          = null;
        $this->_parentEntity    = null;
        $this->_meta            = null;
        $this->_attribute       = null;
        $this->_referenceClass  = null;

        return $this;
    }

    /**
     * @return $this
     * @throws \Exception
     */
    protected function _validate()
    {
        if (!$this->_getEntity()) {
            throw new \B2W\SkyHub\Exception\Exception(get_class($this) . ': Entity cant be empty');
        }

        if (is_null($this->_post)) {
            throw new \B2W\SkyHub\Exception\Exception(get_class($this) . ': Post cant be empty');
        }

        if (is_null($this->_meta)) {
            $this->_meta = get_post_meta($this->_post->ID);
        }

        return $this;
    }

    /**
     * @param MapAttribute $attribute
     * @return mixed
     * @throws \Exception
     */
    protected function _fromMapper(MapAttribute $attribute)
    {
        if (!class_exists($attribute->getMapper('db_to_entity'))) {
            return '';
        }

        /** @var DbToEntityAbstract $mapper */
        $name   = $attribute->getMapper('db_to_entity');
        $mapper = new $name();
        $mapper->setPost($this->_post);
        $mapper->setMeta($this->_meta);
        $mapper->setReferenceClass($this);
        $mapper->setParentEntity($this->_entity);
        $mapper->setAttribute($attribute);
        return $mapper->convert();
    }

    /**
     * @param MapAttribute $attribute
     * @return $this
     * @throws \B2W\SkyHub\Exception\Helper\HelperNotFound
     */
    protected function _setValue(MapAttribute $attribute, $value)
    {
        if (!$attribute->getSkyhub() || !$value) {
            return $this;
        }

        $method = $this->_helper()->getSetterMethodName($this->_entity, $attribute->getSkyhub());

        if (!$method) {
            return $this;
        }

        $this->_entity->$method($value);
    }

    /**
     * @param MapAttribute $attribute
     * @return mixed|null
     */
    protected function _getPostValue(MapAttribute $attribute)
    {
        $post       = $this->_post->to_array();
        $postAttr   = $attribute->getWordpress();

        if (isset($post[$postAttr])) {
            return $post[$postAttr];
        }

        if (isset($this->_meta[$postAttr])) {
            return current($this->_meta[$postAttr]);
        }

        $postAttr = trim($postAttr, '_');

        if (isset($this->_meta[$postAttr])) {
            return current($this->_meta[$postAttr]);
        }

        return null;
    }
}
